package de.hft.geotracker.activities

import android.Manifest.permission.ACCESS_FINE_LOCATION
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.location.Location
import android.os.Bundle
import android.os.Looper
import android.view.MenuItem
import android.widget.ArrayAdapter
import android.widget.Spinner
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.app.ActivityCompat.requestPermissions
import com.google.android.gms.location.*
import de.hft.geotracker.GeofenceBroadcastReceiver
import de.hft.geotracker.R
import de.hft.geotracker.data.JWToken
import de.hft.geotracker.retrofit.AuthenticationInterceptor
import de.hft.geotracker.retrofit.GeofenceService
import de.hft.geotracker.retrofit.ValuesUser
import kotlinx.android.synthetic.main.activity_home.*
import okhttp3.OkHttpClient
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.io.BufferedReader
import java.io.InputStreamReader
import java.lang.StringBuilder
import kotlin.properties.Delegates


class MainActivity : AppCompatActivity() {
    lateinit var geofencingClient: GeofencingClient
    lateinit var geofence: Geofence
    lateinit var actionButton: TextView
    var running = false
    lateinit var service : GeofenceService

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setSupportActionBar(findViewById(R.id.my_toolbar))
        setContentView(R.layout.activity_home)

        this.getSharedPreferences("LOCATION", Context.MODE_PRIVATE)
            ?.edit()
            ?.putBoolean("ENABLED", false)
            ?.apply()
        getSharedPreferences("LOCATION", Context.MODE_PRIVATE)
            .registerOnSharedPreferenceChangeListener { sharedPreferences, key ->
                val btnState = sharedPreferences.getBoolean("ENABLED", false)

                println("Buttonstate: $btnState")
                if (btnState) {
                    button_start_stop?.setBackgroundColor(resources.getColor(R.color.logo_blue))
                } else {
                    button_start_stop?.setBackgroundColor(resources.getColor(R.color.colorPrimaryDark))
                    if (running) {
                        button_start_stop.toggle()
                        callStartStop()
                    }
                }
                button_start_stop.isEnabled = btnState
            }

        //Setup geofence
        geofencingClient = LocationServices.getGeofencingClient(this)
        geofence = Geofence.Builder().setRequestId("Test")
            .setCircularRegion(48.3575, 8.9745, 50F)
            .setExpirationDuration(Geofence.NEVER_EXPIRE)
            .setTransitionTypes(Geofence.GEOFENCE_TRANSITION_ENTER or Geofence.GEOFENCE_TRANSITION_EXIT)
            .build()
        geofencingClient.addGeofences(getGeofencingRequest(), geofencePendingIntent)?.run {
            addOnSuccessListener {
                println("Geofence added")
            }
            addOnFailureListener {
                println("Error: " + it.stackTrace.forEach { println(it.toString()) })
            }
        }

        //JWToken lesen
        var fis = openFileInput("JWToken")
        var isr = InputStreamReader(fis)
        val bufferedReader = BufferedReader(isr)
        val stringBuilder = StringBuilder()
        var text : String? = null
        while ({text = bufferedReader.readLine(); text} () != null) {
            stringBuilder.append(text)
        }
        val token = stringBuilder.toString()
        println("Token Main: " + token)
        //Retrofit declaration
        var httpClient = OkHttpClient.Builder()
        val interceptor = AuthenticationInterceptor(token)
        httpClient.addInterceptor(interceptor)
        val builder = Retrofit.Builder()
            .baseUrl("http://plesk.icaotix.de:5000")
            .addConverterFactory(GsonConverterFactory.create())
            .client(httpClient.build())
        val retrofit = builder.build()
        service = retrofit.create(GeofenceService::class.java)
        showUsername()

        val spinner: Spinner = findViewById(R.id.account_spinner)
        // Create an ArrayAdapter using the string array and a default spinner layout
        ArrayAdapter.createFromResource(
            this,
            R.array.accounts, android.R.layout.simple_spinner_item
        ).also { adapter ->
            // Specify the layout to use when the list of choices appears
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
            // Apply the adapter to the spinner
            spinner.adapter = adapter
        }
        actionButton = findViewById(R.id.button_start_stop)
        actionButton.setOnClickListener {
            callStartStop()
        }
    }

    private fun callStartStop() {

        running = !running
        println("StartStop pressed: $running")
        //ToDO call /track Endpoint
    }

    private fun showUsername() {
        val call = service.getUser()
        call.enqueue(object : Callback<ValuesUser> {
            override fun onResponse(call: Call<ValuesUser>, response: Response<ValuesUser>) {
                if(response.isSuccessful) {
                    val firstname = response.body()?.firstname
                    lbl_username.text = firstname
                    println("Body: " + firstname)
                } else {
                    println("Response not successful: ${response.code()}")
                }
            }
            override fun onFailure(call: Call<ValuesUser>, t: Throwable) {
                println("Response 'whoami' failed")
            }
        } )
    }

    private fun getGeofencingRequest(): GeofencingRequest {
        return GeofencingRequest.Builder().apply {
            setInitialTrigger(GeofencingRequest.INITIAL_TRIGGER_ENTER)
            addGeofence(geofence)
        }.build()
    }

    private val geofencePendingIntent: PendingIntent by lazy {
        val intent = Intent(this, GeofenceBroadcastReceiver::class.java)
        // We use FLAG_UPDATE_CURRENT so that we get the same pending intent back when calling
        // addGeofences() and removeGeofences().
        PendingIntent.getBroadcast(this, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT)
    }

    override fun onOptionsItemSelected(item: MenuItem) = when (item.itemId) {
        R.id.settings -> {
            // User chose the "Settings" item, show the app settings UI...
            var intent = Intent(this, Settings::class.java)
            startActivity(intent)
            println("test")
            true
        }
        R.id.logout -> {
            // User chose the "Settings" item, show the app settings UI...
            var intent = Intent(this, Login::class.java)
            startActivity(intent)
            true
        }
        else -> {
            // If we got here, the user's action was not recognized.
            // Invoke the superclass to handle it.
            println("test")
            super.onOptionsItemSelected(item)
        }
    }

    override fun onBackPressed() {
    }

}
